@@ -2,7 +2,7 @@ from __future__ import print_function, unicode_literals
 import logging
 import os
 import sys
-__version__ = '1.19.0'
+__version__ = '1.20.0'
 
 __all__ = ['assemblers', 'belief', 'databases', 'explanation', 'literature',
            'mechlinker', 'preassembler', 'sources', 'tools', 'util']
@@ -23,7 +23,7 @@ import lib2to3.pgen2.driver
 class Lib2to3LoggingModuleShim(object):
     def getLogger(self):
         return logging.getLogger('lib2to3')
-lib2to3.pgen2.driver.logging = Lib2to3LoggingModuleShim()
+lib2to3.pgen2.driver.logging = Lib2to3LoggingModuleShim()  # type: ignore
 logging.getLogger('lib2to3').setLevel(logging.ERROR)
 
 logger = logging.getLogger('indra')
