@@ -50,7 +50,8 @@ class IndraNetAssembler():
 
     def make_model(self, exclude_stmts=None, complex_members=3,
                    graph_type='multi_graph', sign_dict=None,
-                   belief_flattening=None, weight_flattening=None):
+                   belief_flattening=None, weight_flattening=None,
+                   extra_columns=None):
         """Assemble an IndraNet graph object.
 
         Parameters
@@ -106,7 +107,7 @@ class IndraNetAssembler():
         model : IndraNet
             IndraNet graph object.
         """
-        df = self.make_df(exclude_stmts, complex_members)
+        df = self.make_df(exclude_stmts, complex_members, extra_columns)
         if graph_type == 'multi_graph':
             model = IndraNet.from_df(df)
         elif graph_type == 'digraph':
@@ -125,7 +126,8 @@ class IndraNetAssembler():
                             'type.')
         return model
 
-    def make_df(self, exclude_stmts=None, complex_members=3):
+    def make_df(self, exclude_stmts=None, complex_members=3,
+                extra_columns=None):
         """Create a dataframe containing information extracted from assembler's
         list of statements necessary to build an IndraNet.
 
@@ -138,13 +140,17 @@ class IndraNetAssembler():
             data frame. All complexes larger than complex_members will be
             rejected. For accepted complexes, all permutations of their
             members will be added as dataframe records. Default is `3`.
+        extra_columns : list[tuple(str, function)]
+            A list of tuples defining columns to add to the dataframe in
+            addition to the required columns. Each tuple contains the column
+            name and a function to generate a value from a statement.
 
         Returns
         -------
         df : pd.DataFrame
             Pandas DataFrame object containing information extracted from
             statements. It contains the following columns:
-            
+
             *agA_name*
                 The first Agent's name.
             *agA_ns*
@@ -168,11 +174,19 @@ class IndraNetAssembler():
                 The belief score associated with the statement.
             *source_counts*
                 The number of evidences per input source for the statement.
+            *residue*
+                If applicable, the amino acid residue being modified. NaN if
+                if it is unknown or unspecified/not applicable.
+            *position*
+                If applicable, the position of the modified amino acid. NaN
+                if it is unknown or unspecified/not applicable.
             *initial_sign*
                 The default sign (polarity) associated with the given
                 statement if the statement type has implied polarity.
                 To facilitate weighted path finding, the sign is represented
                 as 0 for positive polarity and 1 for negative polarity.
+
+            More columns can be added by providing the extra_columns parameter.
         """
         rows = []
         if exclude_stmts:
@@ -236,6 +250,14 @@ class IndraNetAssembler():
                 agA_ns, agA_id = get_ag_ns_id(agA)
                 agB_ns, agB_id = get_ag_ns_id(agB)
                 stmt_type = type(stmt).__name__
+                try:
+                    res = stmt.residue
+                except AttributeError:
+                    res = None
+                try:
+                    pos = stmt.position
+                except AttributeError:
+                    pos = None
                 row = OrderedDict([
                     ('agA_name', agA.name),
                     ('agB_name', agB.name),
@@ -243,12 +265,17 @@ class IndraNetAssembler():
                     ('agA_id', agA_id),
                     ('agB_ns', agB_ns),
                     ('agB_id', agB_id),
+                    ('residue', res),
+                    ('position', pos),
                     ('stmt_type', stmt_type),
                     ('evidence_count', len(stmt.evidence)),
                     ('stmt_hash', stmt.get_hash(refresh=True)),
                     ('belief', stmt.belief),
                     ('source_counts', _get_source_counts(stmt)),
                     ('initial_sign', sign)])
+                if extra_columns:
+                    for col_name, func in extra_columns:
+                        row[col_name] = func(stmt)
                 rows.append(row)
         df = pd.DataFrame.from_dict(rows)
         df = df.where((pd.notnull(df)), None)
