@@ -2,30 +2,41 @@ from __future__ import absolute_import, print_function, unicode_literals
 from builtins import dict, str
 import uuid
 import logging
+from typing import Mapping
+
 import networkx as nx
 from copy import deepcopy, copy
 import pybel
 import pybel.constants as pc
 from pybel.dsl import *
-from pybel.language import pmod_namespace
-from pybel.utils import citation_dict
+# This is redundant but needed for documentation build
+from pybel.dsl import Entity
+from pybel.language import pmod_mappings, pmod_namespace, activity_mapping
+try:  # this works after pybel pull request #453
+    from pybel.language import citation_dict
+except ImportError: # this works before pybel pull request #453
+    from pybel.utils import citation_dict
 from indra.statements import *
 from indra.databases import hgnc_client
 
 logger = logging.getLogger(__name__)
 
 
-_indra_pybel_act_map = {
-    'kinase': 'kin',
-    'phosphatase': 'phos',
-    'catalytic': 'cat',
-    'gtpbound': 'gtp',
-    'transcription': 'tscript',
-    'gef': 'gef',
-    'gap': 'gap'
+_indra_pybel_act_map: Mapping[str, Entity] = {
+    'activity': activity_mapping['act'],
+    'kinase': activity_mapping['kin'],
+    'phosphatase': activity_mapping['phos'],
+    'catalytic': activity_mapping['cat'],
+    'gtpbound': activity_mapping['gtp'],
+    'transcription': activity_mapping['tscript'],
+    'gef': activity_mapping['gef'],
+    'gap': activity_mapping['gap'],
 }
 
-_pybel_indra_act_map = {v: k for k, v in _indra_pybel_act_map.items()}
+_pybel_indra_act_map: Mapping[Entity, str] = {
+    pybel_activity: indra_key
+    for indra_key, pybel_activity in _indra_pybel_act_map.items()
+}
 
 
 class PybelAssembler(object):
@@ -64,7 +75,7 @@ class PybelAssembler(object):
     >>> pba = PybelAssembler([stmt])
     >>> belgraph = pba.make_model()
     >>> sorted(node.as_bel() for node in belgraph) # doctest:+IGNORE_UNICODE
-    ['p(HGNC:6840 ! MAP2K1)', 'p(HGNC:6871 ! MAPK1)', 'p(HGNC:6871 ! MAPK1, pmod(Ph, Thr, 185))']
+    ['p(HGNC:6840 ! MAP2K1)', 'p(HGNC:6871 ! MAPK1)', 'p(HGNC:6871 ! MAPK1, pmod(go:0006468 ! "protein phosphorylation", Thr, 185))']
     >>> len(belgraph)
     3
     >>> belgraph.number_of_edges()
@@ -372,39 +383,55 @@ class PybelAssembler(object):
         self._add_nodes_edges(stmt.enz, sub_agent, pc.DIRECTLY_INCREASES, stmt)
 
     def _assemble_translocation(self, stmt):
-        #cc = hierarchies['cellular_component']
-        #nuc_uri = cc.find_entity('nucleus')
-        #cyto_uri = cc.find_entity('cytoplasm')
-        #cyto_go = cyto_uri.rsplit('/')[-1]
         pass
 
 
 def belgraph_to_signed_graph(
         belgraph, include_variants=True, symmetric_variant_links=False,
-        include_components=True, symmetric_component_links=False):
+        include_components=True, symmetric_component_links=False,
+        propagate_annotations=False):
+
+    def get_ns(n):
+        # For nodes containing several agents (complex abundance or reaction)
+        # return namespace of the first member
+        if isinstance(n, complex_abundance):
+            return get_ns(n.members[0])
+        if isinstance(n, reaction):
+            return get_ns(n.products[0])
+        return n.namespace
+
+    graph = nx.MultiDiGraph()
+    for n in belgraph.nodes:
+        graph.add_node(n, ns=get_ns(n))
     edge_set = set()
     for u, v, edge_data in belgraph.edges(data=True):
         rel = edge_data.get('relation')
+        pos_edge = \
+            (u, v, ('sign', 0)) + \
+            tuple((key, tuple(entry)) if len(entry) > 1 else tuple(
+                (key, *tuple(entry)))
+                for key, entry in edge_data.get('annotations', {}).items()) \
+            if propagate_annotations else (u, v, ('sign', 0))
+        # Unpack tuple pairs at indices >1 or they'll be in nested tuples
+        rev_pos_edge = (pos_edge[1], pos_edge[0], *pos_edge[2:])
         if rel in pc.CAUSAL_INCREASE_RELATIONS:
-            edge_set.add((u, v, 0))
+            edge_set.add(pos_edge)
         elif rel in pc.HAS_VARIANT and include_variants:
-            edge_set.add((u, v, 0))
+            edge_set.add(pos_edge)
             if symmetric_variant_links:
-                edge_set.add((v, u, 0))
+                edge_set.add(rev_pos_edge)
         elif rel in pc.PART_OF and include_components:
-            edge_set.add((u, v, 0))
+            edge_set.add(pos_edge)
             if symmetric_component_links:
-                edge_set.add((v, u, 0))
+                edge_set.add(rev_pos_edge)
         elif rel in pc.CAUSAL_DECREASE_RELATIONS:
-            edge_set.add((u, v, 1))
+            # Unpack tuples
+            edge_set.add((pos_edge[0], pos_edge[1],
+                          ('sign', 1), *pos_edge[3:]))
         else:
             continue
-    # Turn the tuples into dicts
-    graph = nx.MultiDiGraph()
-    graph.add_edges_from(
-        (u, v, dict(sign=sign))
-        for u, v, sign in edge_set
-    )
+
+    graph.add_edges_from((t[0], t[1], dict(t[2:])) for t in edge_set)
     return graph
 
 
@@ -439,8 +466,9 @@ def _update_edge_data_from_evidence(evidence, edge_data):
 
 def _get_annotations_from_stmt(stmt):
     return {
-        'stmt_hash': stmt.get_hash(refresh=True),
-        'uuid': stmt.uuid
+        'stmt_hash': {stmt.get_hash(refresh=True): True},
+        'uuid': {stmt.uuid: True},
+        'belief': {stmt.belief: True},
     }
 
 
@@ -487,7 +515,13 @@ def _get_agent_node_no_bcs(agent):
             logger.info('Skipping modification of type %s on agent %s',
                         mod.mod_type, agent)
             continue
-        var = pmod(namespace=pc.BEL_DEFAULT_NAMESPACE, name=pybel_mod)
+        pmod_entity = pmod_mappings[pybel_mod]['xrefs'][0]
+
+        var = ProteinModification(
+            namespace=pmod_entity.namespace,
+            name=pmod_entity.name,
+            identifier=pmod_entity.identifier,
+        )
         if mod.residue is not None:
             res = amino_acids[mod.residue]['short_name'].capitalize()
             var[pc.PMOD_CODE] = res
@@ -582,8 +616,7 @@ def _get_agent_activity(agent):
     if ac.activity_type == 'activity':
         return activity()
 
-    pybel_activity = _indra_pybel_act_map[ac.activity_type]
-    return activity(pybel_activity)
+    return activity(**_indra_pybel_act_map[ac.activity_type])
 
 
 def _get_evidence(evidence):
@@ -592,8 +625,8 @@ def _get_evidence(evidence):
     # If there is a PMID, use it as the citation
     if evidence.pmid:
         citation = citation_dict(
-            db=pc.CITATION_TYPE_PUBMED,
-            db_id=evidence.pmid,
+            namespace=pc.CITATION_TYPE_PUBMED,
+            identifier=evidence.pmid,
         )
     # If no PMID, include the interface and source_api for now--
     # in general this should probably be in the annotations for all evidence
@@ -602,22 +635,24 @@ def _get_evidence(evidence):
         cit_id = evidence.source_id or 'Unknown'
         cit_ref_str = '%s:%s' % (cit_source, cit_id)
         citation = citation_dict(
-            db=pc.CITATION_TYPE_OTHER,
-            db_id=cit_ref_str,
+            namespace=pc.CITATION_TYPE_OTHER,
+            identifier=cit_ref_str,
         )
 
     annotations = {
-        'source_hash': evidence.get_source_hash(),
+        'source_hash': {evidence.get_source_hash(): True},
     }
     if evidence.source_api:
-        annotations['source_api'] = evidence.source_api
+        annotations['source_api'] = {evidence.source_api: True}
     if evidence.source_id:
-        annotations['source_id'] = evidence.source_id
+        annotations['source_id'] = {evidence.source_id: True}
     for key, value in evidence.epistemics.items():
-        if key == 'direct':
+        if key == 'direct' or value is None:
             continue
-        annotations[key] = value
-
+        if isinstance(value, (list, set, tuple)):
+            annotations[key] = {v: True for v in value}
+        else:
+            annotations[key] = {value: True}
     return citation, text, annotations
 
 
