@@ -263,8 +263,8 @@ def _get_title_from_article_element(article):
     title = None
     if title_tag is not None:
         title = title_tag.text
-        if title is None and hasattr(title_tag, 'itertext'):
-            title = ' '.join(list(title_tag.itertext()))
+        if hasattr(title_tag, 'itertext'):
+            title = ''.join(list(title_tag.itertext()))
     return title
 
 
