@@ -19,7 +19,7 @@ class BioOntology(IndraOntology):
     # should be incremented to "force" rebuilding the ontology to be consistent
     # with the underlying resource files.
     name = 'bio'
-    version = '1.7'
+    version = '1.12'
 
     def __init__(self):
         super().__init__()
@@ -89,7 +89,9 @@ class BioOntology(IndraOntology):
 
     def add_hgnc_nodes(self):
         from indra.databases import hgnc_client
-        nodes = [(self.label('HGNC', hid), {'name': hname})
+        withdrawns = set(hgnc_client.hgnc_withdrawn)
+        nodes = [(self.label('HGNC', hid),
+                  {'name': hname, 'obsolete': (hid in withdrawns)})
                  for (hid, hname) in hgnc_client.hgnc_names.items()]
         self.add_nodes_from(nodes)
 
@@ -184,7 +186,13 @@ class BioOntology(IndraOntology):
         for ns in namespaces:
             oc = obo_client.OboClient(prefix=ns)
             for db_id, entry in oc.entries.items():
-                nodes.append((self.label(ns.upper(), db_id),
+                label = self.label(ns.upper(), db_id)
+                # There may be more general ways of doing this but this
+                # handles the special case of BFO nodes in the EFO
+                # ontology.
+                if ns == 'efo' and db_id.startswith('BFO'):
+                    label = db_id
+                nodes.append((label,
                               {'name': entry['name']}))
         self.add_nodes_from(nodes)
 
@@ -192,19 +200,25 @@ class BioOntology(IndraOntology):
         from indra.databases import obo_client
         namespaces = ['go', 'efo', 'hp', 'doid', 'chebi']
         edges = []
+        # Mapping various source relation types to standardized ones
+        # in this ontology graph
         rel_mappings = {
             'xref': 'xref',
             'isa': 'isa',
             'partof': 'partof',
             'is_a': 'isa',
             'part_of': 'partof',
-            # These are for ChEBI: identical to the old behavior but it might
-            # make sense to add other relations here too
-            'is_conjugate_acid_of': 'isa',
+            'has_part': 'partof',
+            # These are specifically to map ChEBI relations
             'has_functional_parent': 'isa',
             'has_parent_hydride': 'isa',
             'has_role': 'isa'
         }
+        # The source and target for these relations need to be reversed
+        # when adding to the graph
+        reverse_rel = {
+            'has_part',
+        }
         for ns in namespaces:
             oc = obo_client.OboClient(prefix=ns)
             for db_id, entry in oc.entries.items():
@@ -213,10 +227,22 @@ class BioOntology(IndraOntology):
                     mapped_rel = rel_mappings.get(rel)
                     if not mapped_rel:
                         continue
+                    source_label = self.label(ns.upper(), db_id)
+                    if ns == 'efo' and db_id.startswith('BFO'):
+                        source_label = db_id
                     for target in targets:
-                        edges.append((self.label(ns.upper(), db_id),
-                                      self.label(ns.upper(), target),
-                                      {'type': mapped_rel}))
+                        target_label = self.label(ns.upper(), target)
+                        if ns == 'efo' and target.startswith('BFO'):
+                            target_label = target
+                        if rel in reverse_rel:
+                            av = (target_label,
+                                  source_label,
+                                  {'type': mapped_rel})
+                        else:
+                            av = (source_label,
+                                  target_label,
+                                  {'type': mapped_rel})
+                        edges.append(av)
         self.add_edges_from(edges)
 
     def add_chemical_xrefs(self):
