@@ -224,7 +224,7 @@ def update_hmdb_chebi_map():
     url = 'http://www.hmdb.ca/system/downloads/current/hmdb_metabolites.zip'
     fname = os.path.join(path, 'hmdb_metabolites.zip')
     logger.info('Downloading %s' % url)
-    #urlretrieve(url, fname)
+    urlretrieve(url, fname)
     mappings = []
     with ZipFile(fname) as input_zip:
         with input_zip.open('hmdb_metabolites.xml') as fh:
@@ -246,7 +246,11 @@ def update_hmdb_chebi_map():
                 elif event == 'end' and \
                         elem.tag == '{%s}metabolite' % ns['hmdb']:
                     if hmdb_id and chebi_id:
-                        print(hmdb_id, chebi_id)
+                        name = chebi_client.get_chebi_name_from_id(chebi_id)
+                        if not name:
+                            print('Likely invalid ChEBI mapping: ',
+                                  hmdb_id, chebi_id)
+                            continue
                         mappings.append([hmdb_id, chebi_id])
                 elem.clear()
     fname = os.path.join(path, 'hmdb_to_chebi.tsv')
@@ -449,13 +453,21 @@ def update_famplex():
 
 def update_grounding_map():
     famplex_gmap = os.path.join(path, 'famplex', 'grounding_map.csv')
-    covid_gmap = os.path.join(path, 'grounding', 'covid_grounding.csv')
     famplex_rows = list(read_unicode_csv(famplex_gmap))
     row_len = len(famplex_rows[0])
-    covid_rows = list(read_unicode_csv(covid_gmap))
-    covid_rows = [r + [''] * (row_len - len(r))
-                  for r in covid_rows]
-    all_rows = famplex_rows + covid_rows
+    extra_rows = []
+    # read in json file containing filenames for non-famplex grounding maps
+    with open(os.path.join(path, 'grounding', 'extra_gmap_files.json')) as f:
+        extra_gm_files = json.load(f)
+    # Add non-famplex grounding map rows, adding blank values to synchronize
+    # the number of columns with the number in the famplex grounding map
+    for gm_filename in extra_gm_files:
+        gmap = os.path.join(path, 'grounding', gm_filename)
+        new_rows = list(read_unicode_csv(gmap))
+        new_rows = [r + [''] * (row_len - len(r))
+                    for r in new_rows]
+        extra_rows.extend(new_rows)
+    all_rows = famplex_rows + extra_rows
     grounding_map = os.path.join(path, 'grounding', 'grounding_map.csv')
     write_unicode_csv(grounding_map, all_rows)
 
@@ -608,7 +620,8 @@ def update_doid():
 def update_efo():
     """Update experimental factor ontology."""
     url = 'https://www.ebi.ac.uk/efo/efo.obo'
-    OboClient.update_resource(path, url, 'efo', remove_prefix=True)
+    OboClient.update_resource(path, url, 'efo', remove_prefix=True,
+                              allowed_external_ns={'BFO'})
 
 
 def update_hpo():
@@ -681,12 +694,17 @@ def update_biomappings():
     biomappings = defaultdict(list)
     mappings = load_mappings()
     predictions = load_predictions()
+    exclude_ns = {'kegg.pathway'}
     for mappings, mapping_type in ((mappings, 'curated'),
-                                  (predictions, 'predicted')):
+                                   (predictions, 'predicted')):
         for mapping in mappings:
             # We skip anything that isn't an exact match
             if mapping['relation'] != 'skos:exactMatch':
                 continue
+            # Skip excluded name spaces that aren't relevant here
+            if mapping['source prefix'] in exclude_ns or \
+                    mapping['target prefix'] in exclude_ns:
+                continue
             # We only accept curated mappings for NCIT
             if mapping_type == 'predicted' and \
                     (mapping['source prefix'] == 'ncit' or
@@ -704,7 +722,7 @@ def update_biomappings():
             biomappings[(source_ns, source_id, mapping['source name'])].append(
                 (target_ns, target_id, mapping['target name']))
             biomappings[(target_ns, target_id, mapping['target name'])].append(
-                (source_ns, source_id, mapping['target name']))
+                (source_ns, source_id, mapping['source name']))
 
     mesh_mappings = {k: v for k, v in biomappings.items()
                      if k[0] == 'MESH'}
