@@ -12,6 +12,8 @@ from indra.statements import stmts_from_json, get_statement_by_name, \
 from indra.sources.indra_db_rest.util import submit_query_request, \
     submit_statement_request
 from indra.sources.indra_db_rest.exceptions import IndraDBRestResponseError
+from indra.util.statement_presentation import get_available_source_counts, \
+    get_available_ev_counts, standardize_counts
 
 logger = logging.getLogger(__name__)
 
@@ -49,6 +51,10 @@ class IndraDBRestProcessor(object):
         Select the maximum number of statements to return. When set less than
         1000 the effect is much the same as setting persist to false, and will
         guarantee a faster response. Default is None.
+    use_obtained_counts : Optional[bool]
+        If set to True, the statement evidence counts and source counts are
+        calculated based on the actual obtained statements and evidences
+        rather than the counts provided by the DB API. Default: False.
 
     Attributes
     ----------
@@ -64,6 +70,8 @@ class IndraDBRestProcessor(object):
         self.__evidence_counts = {}
         self.__source_counts = {}
 
+        self.use_obtained_counts = kwargs.pop('use_obtained_counts', False)
+
         # Define the basic generic defaults.
         default_api_params = dict(timeout=None, ev_limit=10, best_first=True,
                                   tries=2, max_stmts=None)
@@ -80,7 +88,6 @@ class IndraDBRestProcessor(object):
         # specified by the user.
         kwargs.update((k, kwargs.get(k, default_api_params[k]))
                       for k in default_api_params.keys())
-
         self._run(*args, **kwargs)
         return
 
@@ -90,7 +97,7 @@ class IndraDBRestProcessor(object):
 
     def get_ev_count_by_hash(self, stmt_hash):
         """Get the total evidence count for a statement hash."""
-        return self.__evidence_counts.get(str(stmt_hash), 0)
+        return self.__evidence_counts.get(stmt_hash, 0)
 
     def get_source_counts(self):
         """Get the source counts as a dict per statement hash."""
@@ -129,11 +136,10 @@ class IndraDBRestProcessor(object):
     def _merge_json(self, stmt_json, ev_counts, source_counts):
         """Merge these statement jsons with new jsons."""
         # Where there is overlap, there _should_ be agreement.
-        self.__evidence_counts.update(ev_counts)
+        self.__evidence_counts.update(standardize_counts(ev_counts))
 
         # We turn source counts into an int-keyed dict and update it that way
-        self.__source_counts.update({int(k): v
-                                     for k, v in source_counts.items()})
+        self.__source_counts.update(standardize_counts(source_counts))
 
         for k, sj in stmt_json.items():
             if k not in self.__statement_jsons:
@@ -148,6 +154,9 @@ class IndraDBRestProcessor(object):
     def _compile_statements(self):
         """Generate statements from the jsons."""
         self.statements = stmts_from_json(self.__statement_jsons.values())
+        if self.use_obtained_counts:
+            self.__source_counts = get_available_source_counts(self.statements)
+            self.__evidence_counts = get_available_ev_counts(self.statements)
 
     def _unload_and_merge_resp(self, resp):
         resp_dict = resp.json(object_pairs_hook=OrderedDict)
@@ -382,7 +391,7 @@ class IndraDBRestSearchProcessor(IndraDBRestProcessor):
                            "statement load to complete." % dt.total_seconds())
             ret = False
         else:
-            logger.info("Waited %0.3f seconds for statements to finish"
+            logger.info("Waited %0.3f seconds for statements to finish "
                         "loading." % dt.total_seconds())
             ret = True
         return ret
