@@ -3,7 +3,8 @@ __all__ = ['TasProcessor']
 import logging
 from indra.statements import Inhibition, Agent, Evidence
 from indra.statements.validate import assert_valid_db_refs
-from indra.ontology.standardize import standardize_name_db_refs
+from indra.ontology.standardize import standardize_name_db_refs, \
+    get_standard_agent
 from indra.databases import hgnc_client, chembl_client, lincs_client
 
 
@@ -91,15 +92,11 @@ class TasProcessor(object):
         return drugs
 
     def _extract_protein(self, name, gene_id):
-        refs = {'EGID': gene_id}
+        db_refs = {'EGID': gene_id}
         hgnc_id = hgnc_client.get_hgnc_from_entrez(gene_id)
         if hgnc_id is not None:
-            refs['HGNC'] = hgnc_id
-        standard_name, db_refs = standardize_name_db_refs(refs)
-        if standard_name:
-            name = standard_name
-        assert_valid_db_refs(db_refs)
-        return Agent(name, db_refs=db_refs)
+            db_refs['HGNC'] = hgnc_id
+        return get_standard_agent(name, db_refs=db_refs)
 
     def _make_evidences(self, class_min, references):
         evidences = []
