@@ -29,11 +29,11 @@ def test_get_sm_name():
 
 def test_get_sm_refs():
     sm_refs = lc.get_small_molecule_refs('10001')
-    assert sm_refs.get('HMS-LINCS') == '10001'
-    assert sm_refs.get('PUBCHEM') == '160355'
-    assert sm_refs.get('CHEMBL') == '14762'
+    assert sm_refs.get('HMS-LINCS') == '10001', sm_refs
+    assert sm_refs.get('PUBCHEM') == '160355', sm_refs
+    assert sm_refs.get('CHEMBL') == 'CHEMBL14762', sm_refs
 
-    sm_ref = lc.get_small_molecule_refs('10001-101')
-    assert sm_refs.get('HMS-LINCS') == '10001'
-    assert sm_refs.get('PUBCHEM') == '160355'
-    assert sm_refs.get('CHEMBL') == '14762'
+    sm_refs = lc.get_small_molecule_refs('10001-101')
+    assert sm_refs.get('HMS-LINCS') == '10001-101', sm_refs
+    assert sm_refs.get('PUBCHEM') == '160355', sm_refs
+    assert sm_refs.get('CHEMBL') == 'CHEMBL14762', sm_refs
