@@ -4,6 +4,7 @@ import networkx as nx
 from indra.explanation.pathfinding.pathfinding import bfs_search, \
     shortest_simple_paths, bfs_search_multiple_nodes, open_dijkstra_search, \
     simple_paths_with_constraints
+from indra.explanation.pathfinding.util import get_subgraph
 from indra.explanation.model_checker.model_checker import \
     signed_edges_to_signed_nodes
 
@@ -180,6 +181,39 @@ def test_bfs():
         error = True
     assert error
 
+    # Test edge filter
+    # With belief cutoff at 0.75, we should remove:
+    # ('A2', 'B1'), ('A3', 'B2'), ('A4', 'B2'), ('B2', 'C1'), ('B3', 'C1')
+    def _filter_func(g, u, v):
+        return g.edges[u, v]['belief'] > 0.75
+
+    expected_paths = {('D1', 'C1'), ('D1', 'C1', 'B1'),
+                      ('D1', 'C1', 'B1', 'A1'),
+                      ('D1', 'C1', 'B1', 'A1', 'Z1')}
+    gen = bfs_search(g=dg, source_node='D1', depth_limit=5, reverse=True,
+                     edge_filter=_filter_func)
+    paths = list(gen)
+    assert len(paths) == len(expected_paths), f'Expected ' \
+                                              f'{len(expected_paths)}, ' \
+                                              f'got {len(paths)} paths'
+    assert set(paths) == expected_paths
+
+    # Test edge_filter and allowed_edges
+    def _alwd_edges(u, v):
+        # Edges should not be reversed!
+        return (u, v) in {('C1', 'D1'), ('B1', 'C1'), ('A1', 'B1')}
+    expected_paths = {('D1', 'C1'), ('D1', 'C1', 'B1'),
+                      ('D1', 'C1', 'B1', 'A1')}
+    some_new_gen = bfs_search(g=dg, source_node='D1', depth_limit=5,
+                              reverse=True, edge_filter=_filter_func,
+                              allow_edge=_alwd_edges, hashes=[123456897],
+                              strict_mesh_id_filtering=True)
+    paths = list(some_new_gen)
+    assert len(paths) == len(expected_paths), f'Expected ' \
+                                              f'{len(expected_paths)}, ' \
+                                              f'got {len(paths)} paths'
+    assert set(paths) == expected_paths
+
 
 def test_signed_bfs():
     seg, sng, all_ns = _setup_signed_graph()
@@ -554,3 +588,29 @@ def test_simple_paths_with_constraints():
     paths = tuple([tuple(p) for p in simple_paths_with_constraints(
         dg, 'A3', 'D1', filter_func=_isB)])
     assert len(paths) == 0
+
+
+def test_get_subgraph_custom_filter():
+
+    def filter_to_high_belief(g, u, v, *args):
+        if args:
+            edge = g[u][v][args[0]]
+        else:
+            edge = g[u][v]
+        return edge['belief'] > 0.5
+
+    # Signed graph
+    sg, _, _ = _setup_signed_graph()
+    assert len(sg.edges) == 12
+    assert ('A4', 'B2', 1) in sg.edges
+    filtered_sg = get_subgraph(sg, filter_to_high_belief)
+    assert len(filtered_sg.edges) == 10
+    assert ('A4', 'B2', 1) not in filtered_sg.edges
+
+    # Unsigned graph
+    ug, _ = _setup_unsigned_graph()
+    assert len(ug.edges) == 9
+    assert ('A4', 'B2') in ug.edges
+    filtered_ug = get_subgraph(ug, filter_to_high_belief)
+    assert len(filtered_ug.edges) == 7
+    assert ('A4', 'B2') not in filtered_ug.edges
