@@ -3,6 +3,7 @@
 """Tests for the PyBEL assembler."""
 
 import json
+
 import networkx as nx
 import pybel.constants as pc
 from pybel.dsl import abundance, activity, bioprocess, \
@@ -101,7 +102,11 @@ def test_modification_with_evidences():
     braf_kin = Agent('BRAF', activity=ActivityCondition('kinase', True),
                      db_refs={'HGNC': '1097', 'UP': 'P15056'})
     mek = Agent('MAP2K1', db_refs={'HGNC': '6840', 'UP': 'Q02750'})
-    evidence = Evidence(source_api='test', text='evidence text', pmid='1234')
+    evidence = Evidence(source_api='test', text='evidence text', pmid='1234', epistemics={
+        'dummy': ['a', 'b'],
+        'scalar': 'yes',
+        'missing': None,
+    })
     stmt = Phosphorylation(braf_kin, mek, 'S', '218', evidence=evidence)
     pba = pa.PybelAssembler([stmt])
     belgraph = pba.make_model()
@@ -119,9 +124,15 @@ def test_modification_with_evidences():
         pc.CITATION_IDENTIFIER: '1234',
     }
     assert 'source_api' in edge_data[pc.ANNOTATIONS]
-    assert edge_data[pc.ANNOTATIONS]['source_api'] == 'test'
+    assert 'test' in edge_data[pc.ANNOTATIONS]['source_api']
     assert 'source_id' not in edge_data[pc.ANNOTATIONS]
     assert 'source_hash' in edge_data[pc.ANNOTATIONS]
+    assert 'dummy' in edge_data[pc.ANNOTATIONS]
+    assert 'a' in edge_data[pc.ANNOTATIONS]['dummy']
+    assert 'b' in edge_data[pc.ANNOTATIONS]['dummy']
+    assert 'scalar' in edge_data[pc.ANNOTATIONS]
+    assert 'yes' in edge_data[pc.ANNOTATIONS]['scalar']
+    assert 'missing' not in edge_data[pc.ANNOTATIONS]
 
 
 def test_modification_with_mutation():
@@ -148,10 +159,11 @@ def test_activation():
     hash2 = stmt2.get_hash(refresh=True)
     edge1 = {
         pc.RELATION: pc.INCREASES,
-        pc.OBJECT: {pc.MODIFIER: pc.ACTIVITY},
+        pc.OBJECT: activity(),
         pc.ANNOTATIONS: {
-            'stmt_hash': hash1,
-            'uuid': stmt1.uuid,
+            'stmt_hash': {hash1: True},
+            'uuid': {stmt1.uuid: True},
+            'belief': {stmt1.belief: True},
         },
     }
     edge2 = {
@@ -159,8 +171,9 @@ def test_activation():
         pc.SUBJECT: activity('kin'),
         pc.OBJECT: activity('kin'),
         pc.ANNOTATIONS: {
-            'stmt_hash': hash2,
-            'uuid': stmt2.uuid,
+            'stmt_hash': {hash2: True},
+            'uuid': {stmt2.uuid: True},
+            'belief': {stmt2.belief: True},
         },
     }
     for stmt, edge in ((stmt1, edge1), (stmt2, edge2)):
@@ -189,16 +202,17 @@ def test_direct_activation():
     hash2 = stmt2.get_hash(refresh=True)
     edge1 = {
         pc.RELATION: pc.DIRECTLY_INCREASES,
-        pc.OBJECT: {pc.MODIFIER: pc.ACTIVITY},
+        pc.OBJECT: activity(),
         pc.EVIDENCE: 'No evidence text.',
         pc.CITATION: {
             pc.CITATION_DB: pc.CITATION_TYPE_PUBMED,
             pc.CITATION_IDENTIFIER: '1234',
         },
         pc.ANNOTATIONS: {
-            'stmt_hash': hash1,
-            'source_hash': stmt1_ev.get_source_hash(),
-            'uuid': stmt1.uuid,
+            'stmt_hash': {hash1: True},
+            'source_hash': {stmt1_ev.get_source_hash(): True},
+            'uuid': {stmt1.uuid: True},
+            'belief': {stmt1.belief: True},
         },
     }
     edge2 = {
@@ -211,9 +225,10 @@ def test_direct_activation():
             pc.CITATION_IDENTIFIER: '1234',
         },
         pc.ANNOTATIONS: {
-            'stmt_hash': hash2,
-            'source_hash': stmt1_ev.get_source_hash(),
-            'uuid': stmt2.uuid,
+            'stmt_hash': {hash2: True},
+            'source_hash': {stmt1_ev.get_source_hash(): True},
+            'uuid': {stmt2.uuid: True},
+            'belief': {stmt2.belief: True},
         },
     }
     for stmt, expected_edge in ((stmt1, edge1), (stmt2, edge2)):
@@ -238,8 +253,9 @@ def test_inhibition():
         pc.SUBJECT: activity('kin'),
         pc.OBJECT: activity('kin'),
         pc.ANNOTATIONS: {
-            'stmt_hash': stmt_hash,
-            'uuid': stmt.uuid,
+            'stmt_hash': {stmt_hash: True},
+            'uuid': {stmt.uuid: True},
+            'belief': {stmt.belief: True},
         },
     }
     pba = pa.PybelAssembler([stmt])
@@ -308,8 +324,9 @@ def test_gef():
         pc.SUBJECT: activity('gef'),
         pc.OBJECT: activity('gtp'),
         pc.ANNOTATIONS: {
-            'stmt_hash': stmt_hash,
-            'uuid': stmt.uuid,
+            'stmt_hash': {stmt_hash: True},
+            'uuid': {stmt.uuid: True},
+            'belief': {stmt.belief: True},
         },
     }
     assert edge_data == edge, edge_data
@@ -340,8 +357,9 @@ def test_gap():
         pc.SUBJECT: activity('gap'),
         pc.OBJECT: activity('gtp'),
         pc.ANNOTATIONS: {
-            'stmt_hash': stmt_hash,
-            'uuid': stmt.uuid,
+            'stmt_hash': {stmt_hash: True},
+            'uuid': {stmt.uuid: True},
+            'belief': {stmt.belief: True},
         },
     }
     assert edge_data == edge, edge_data
@@ -443,9 +461,11 @@ def test_autophosphorylation():
     edge_dicts = list(belgraph.get_edge_data(egfr_dsl,
                                              egfr_phos_node).values())
     assert {pc.RELATION: pc.DIRECTLY_INCREASES,
-            pc.ANNOTATIONS: {'stmt_hash': stmt_hash,
-                             'uuid': stmt.uuid}} \
-        in edge_dicts
+            pc.ANNOTATIONS: {
+                'stmt_hash': {stmt_hash: True},
+                'uuid': {stmt.uuid: True},
+                'belief': {stmt.belief: True},
+            }} in edge_dicts
 
     # Test an autophosphorylation with a bound condition
     tab1 = Agent('TAB1', db_refs={'HGNC': id('TAB1')})
@@ -487,8 +507,9 @@ def test_bound_condition():
              pc.RELATION: pc.DIRECTLY_INCREASES,
              pc.OBJECT: activity('gtp'),
              pc.ANNOTATIONS: {
-                 'stmt_hash': stmt_hash,
-                 'uuid': stmt.uuid,
+                 'stmt_hash': {stmt_hash: True},
+                 'uuid': {stmt.uuid: True},
+                 'belief': {stmt.belief: True},
              },
         },
     )
@@ -513,8 +534,9 @@ def test_transphosphorylation():
     assert edge_data == {
         pc.RELATION: pc.DIRECTLY_INCREASES,
         pc.ANNOTATIONS: {
-            'stmt_hash': stmt_hash,
-            'uuid': stmt.uuid,
+            'stmt_hash': {stmt_hash: True},
+            'uuid': {stmt.uuid: True},
+            'belief': {stmt.belief: True},
         },
     }, edge_data
 
@@ -602,3 +624,34 @@ def test_no_activity_on_bioprocess():
 
     _, _, e = list(belgraph.edges(data=True))[0]
     assert pc.OBJECT not in e
+
+
+def test_belgraph_to_signed_graph():
+    braf_no_act = Agent('BRAF', db_refs={'HGNC': '1097', 'UP': 'P15056'})
+    mek = Agent('MAP2K1', db_refs={'HGNC': '6840', 'UP': 'Q02750'})
+    stmt = Activation(braf_no_act, mek)
+    hsh = stmt.get_hash(refresh=True)
+
+    pba = pa.PybelAssembler([stmt])
+    belgraph = pba.make_model()
+    pb_seg = pa.belgraph_to_signed_graph(belgraph, propagate_annotations=True)
+
+    assert len(pb_seg.edges) == 1
+
+    edge = (braf_dsl, map2k1_dsl, 0)
+    assert edge in pb_seg.edges
+
+    edge_dict = pb_seg.edges.get(edge)
+    assert edge_dict
+
+    assert 'stmt_hash' in edge_dict
+    assert isinstance(edge_dict['stmt_hash'], int)
+    assert hsh == edge_dict['stmt_hash']
+
+    assert 'uuid' in edge_dict
+    assert isinstance(edge_dict['uuid'], str)
+    assert stmt.uuid == edge_dict['uuid']
+
+    assert 'belief' in edge_dict
+    assert isinstance(edge_dict['belief'], (float, int))
+    assert stmt.belief == edge_dict['belief']
