@@ -20,6 +20,7 @@ from indra.databases import hgnc_client
 from indra.ontology.bio import bio_ontology
 from indra.ontology.world import world_ontology
 from indra.preassembler import Preassembler, flatten_evidence
+from indra.resources import get_resource_path
 
 
 logger = logging.getLogger(__name__)
@@ -376,7 +377,7 @@ def map_sequence(stmts_in, do_methionine_offset=True,
 @register_pipeline
 def run_preassembly(stmts_in, return_toplevel=True, poolsize=None,
                     size_cutoff=None, belief_scorer=None, ontology=None,
-                    matches_fun=None, refinement_fun=None, refinement_ns=None,
+                    matches_fun=None, refinement_fun=None,
                     flatten_evidence=False, flatten_evidence_collect_from=None,
                     normalize_equivalences=False, normalize_opposites=False,
                     normalize_ns='WM', run_refinement=True, filters=None,
@@ -412,9 +413,6 @@ def run_preassembly(stmts_in, return_toplevel=True, poolsize=None,
     refinement_fun : Optional[function]
         A function to override the built-in refinement_of function of
         statements.
-    refinement_ns : Optional[set]
-        A set of name spaces over which refinements should be constructed.
-        If not provided, all name spaces are considered.
     flatten_evidence : Optional[bool]
         If True, evidences are collected and flattened via supports/supported_by
         links. Default: False
@@ -432,13 +430,16 @@ def run_preassembly(stmts_in, return_toplevel=True, poolsize=None,
     normalize_ns : Optional[str]
         The name space with respect to which equivalences and opposites are
         normalized.
-    filters : Optional[list[function]]
-        A list of function handles that define filter functions on
-        possible statement refinements. Each function takes
-        a stmts_by_hash dict as its input and returns a dict
-        of possible refinements where the keys are statement hashes
-        and the values are sets of statement hashes that the
-        key statement possibly refines.
+    filters : Optional[list[:py:class:indra.preassembler.refinement.RefinementFilter]]
+        A list of RefinementFilter classes that implement filters on
+        possible statement refinements. For details on how to
+        construct such a filter, see the documentation of
+        :py:class:`indra.preassembler.refinement.RefinementFilter`.
+        If no user-supplied filters are provided, the default ontology-based
+        filter is applied. If a list of filters is provided here, the
+        :py:class:`indra.preassembler.refinement.OntologyRefinementFilter`
+        isn't appended by default, and should be added by the user, if
+        necessary. Default: None
     save : Optional[str]
         The name of a pickle file to save the results (stmts_out) into.
     save_unique : Optional[str]
@@ -453,8 +454,7 @@ def run_preassembly(stmts_in, return_toplevel=True, poolsize=None,
     use_ontology = ontology if ontology is not None else bio_ontology
     be = BeliefEngine(scorer=belief_scorer, matches_fun=matches_fun)
     pa = Preassembler(use_ontology, stmts_in, matches_fun=matches_fun,
-                      refinement_fun=refinement_fun,
-                      refinement_ns=refinement_ns)
+                      refinement_fun=refinement_fun)
     if normalize_equivalences:
         logger.info('Normalizing equals on %d statements' % len(pa.stmts))
         pa.normalize_equivalences(normalize_ns)
@@ -2083,3 +2083,45 @@ def filter_complexes_by_size(stmts_in, members_allowed=5):
             stmts_out.append(stmt)
     logger.info('%d statements after filter...' % len(stmts_out))
     return stmts_out
+
+
+def _load_db_refs_map():
+    filepath = get_resource_path('db_refs_map.csv')
+    rows = read_unicode_csv(filepath)
+    db_refs_map = {}
+    for (db_ns, old_id, new_id) in rows:
+        db_refs_map[(db_ns, old_id)] = new_id
+    return db_refs_map
+
+
+@register_pipeline
+def map_db_refs(stmts_in, db_refs_map=None):
+    """Update entries in db_refs to those provided in db_refs_map.
+
+    Parameters
+    ----------
+    stmts_in : list[indra.statements.Statement]
+        A list of INDRA Statements to update db_refs in.
+    db_refs_map : Optional[dict]
+        A dictionary where each key is a tuple (db_ns, db_id) representing old
+        db_refs pair that has to be updated and each value is a new db_id to
+        replace the old value with. If not provided, the default db_refs_map
+        will be loaded.
+    """
+    if not db_refs_map:
+        db_refs_map = _load_db_refs_map()
+    stmts_out = []
+
+    def update_agent_db_refs(ag_db_refs, db_refs_map):
+        for (db_ns, old_db_id), new_id in db_refs_map.items():
+            if ag_db_refs.get(db_ns) == old_db_id:
+                ag_db_refs[db_ns] = new_id
+        return ag_db_refs
+
+    for stmt in stmts_in:
+        new_stmt = deepcopy(stmt)
+        for ag in new_stmt.agent_list():
+            if ag is not None:
+                ag.db_refs = update_agent_db_refs(ag.db_refs, db_refs_map)
+        stmts_out.append(new_stmt)
+    return stmts_out
